if AleaUI_GUI then return end
local C = _G['AleaGUI_PrototypeLib']

C.colorFrames = {}

local current

local function func_1(restore)
	local newR, newG, newB, newA;
	if restore then
		newR, newG, newB, newA = unpack(restore);
	else	
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB();
	end
	
	current._OnClick(_, newR, newG, newB,newA)
	current:SetBackdropColor(newR, newG, newB,newA or 1) -- 
	
	C:GetRealParent(current):RefreshData()
end


local function ShowColorPicker(r, g, b, a, hasalpha, f)
 
 current = f
 
 ColorPickerFrame:SetColorRGB(r,g,b);

 ColorPickerFrame.hasOpacity = hasalpha
 
 ColorPickerFrame.opacity = a;
 
 ColorPickerFrame.previousValues = {r,g,b,a};
 
 ColorPickerFrame.func = func_1
 ColorPickerFrame.opacityFunc = func_1
 ColorPickerFrame.cancelFunc = func_1

 ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
 ColorPickerFrame:Show();
end


local colorpicker = CreateFrame("Frame", nil, UIParent)
colorpicker:SetSize(300, 200)
colorpicker:SetPoint("CENTER")




local function Update(self, panel, opts)
	
	self.free = false
	self:SetParent(panel)
	self:Show()	
	
	self:SetDescription(opts.desc)
	self:SetName(opts.name)
	
	self:UpdateColor(opts)
	self:UpdateState(opts.set, opts.get)
end

local function Remove(self)
	self.free = true
	self:Hide()	
end

local function SetName(self, name)
	self.main._rname = name
	self.main.text:SetText(name)
end

local function SetDescription(self, text)
	self.main.desc = text
end

local function UpdateState(self, func, get)
	
	self.main._OnClick = func
	self.main._OnShow = get

end

local function UpdateColor(self, opts)

	self.main.hasAlpha = opts.hasAlpha
	
	local r,g,b,a = opts.get()
	
	self.main.texture:SetTexture(r,g,b, self.main.hasAlpha and ( a or 1 ) or 1)

end

local function CreateCoreButton(parent)
	local f = CreateFrame('Button', nil, parent) --"UICheckButtonTemplate"

	f:SetFrameLevel(parent:GetFrameLevel() + 1)
	f:SetSize(21, 21)

	local colorSwatch = f:CreateTexture(nil, "OVERLAY")
	colorSwatch:SetWidth(16)
	colorSwatch:SetHeight(16)
	colorSwatch:SetTexture("Interface\\ChatFrame\\ChatFrameColorSwatch")
	colorSwatch:SetPoint("CENTER")

	local texture = f:CreateTexture(nil, "BACKGROUND", 0)
	texture:SetWidth(13)
	texture:SetHeight(13)
	texture:SetTexture(1, 1, 1)
	texture:SetPoint("CENTER", colorSwatch)
	texture:Show()
	
	local checkers = f:CreateTexture(nil, "BACKGROUND", -1)
	checkers:SetWidth(11)
	checkers:SetHeight(11)
	checkers:SetTexture("Tileset\\Generic\\Checkers")
	checkers:SetTexCoord(.25, 0, 0.5, .25)
--	checkers:SetDesaturated(true)
	checkers:SetVertexColor(1, 1, 1, 0.75)
	checkers:SetPoint("CENTER", colorSwatch)
	checkers:Show()
	
	f:SetBackdrop({
		bgFile = "",
		edgeFile = [=[Interface\ChatFrame\ChatFrameBackground]=], 
		edgeSize = 2,
		insets = {top = 0, left = 0, bottom = 0, right = 0},
		})
	f:SetBackdropBorderColor(unpack(C.button_border_color_ondown)) -- 
	
	f:SetScript("OnEnter", function(self)
		self:SetBackdropBorderColor(unpack(C.button_border_color_onup)) -- 		
		C.Tooltip(self, self._rname, self.desc, "show")
	end)
	f:SetScript("OnLeave", function(self)
		self:SetBackdropBorderColor(unpack(C.button_border_color_ondown)) -- 
		C.Tooltip(self, self._rname, self.desc, "hide")
	end)
	
	
	f:SetScript("OnClick", function(self)
		local r,g,b,a = self._OnShow()	
		if self.hasAlpha then
			ShowColorPicker(r,g,b,(a or 1), self.hasAlpha, self)
		else
			ShowColorPicker(r,g,b,1, self.hasAlpha, self)
		end
	end)

	local text = f:CreateFontString(nil, 'OVERLAY', "GameFontHighlight")
	text:SetPoint("LEFT", f, "RIGHT", 3 , 0)
--	text:SetFont(STANDARD_TEXT_FONT, 12, "OUTLINE")
	text:SetTextColor(1, 1, 1)
	text:SetJustifyH("LEFT")
	text:SetWordWrap(false)
	
	f.mouseover = CreateFrame("Frame", nil, f)
	f.mouseover:SetFrameLevel(f:GetFrameLevel()-1)
	f.mouseover:SetSize(1,1)
	f.mouseover:SetPoint("TOPLEFT", text, "TOPLEFT", -3, 3)
	f.mouseover:SetPoint("BOTTOMRIGHT", text, "BOTTOMRIGHT", 3, -3)
	f.mouseover:SetScript("OnEnter", function(self)
--		self:GetParent():SetBackdropBorderColor(unpack(C.button_border_color_onup)) -- 		
		C.Tooltip(self, self:GetParent()._rname, self:GetParent().desc, "show")
	end)
	f.mouseover:SetScript("OnLeave", function(self)
--		self:GetParent():SetBackdropBorderColor(unpack(C.button_border_color_ondown)) -- 
	
		C.Tooltip(self, self:GetParent()._rname, self:GetParent().desc, "hide")
	end)
	
	f.text = text
	f.texture = colorSwatch
	
	return f
end

function C:CreateColorFrame()
	
	for i=1, #C.colorFrames do
		if C.colorFrames[i].free then
			return C.colorFrames[i]
		end
	end
	
	local f = CreateFrame("Frame", nil, UIParent)
	f:SetSize(180, 45)
	f.free = true
	
	f.main = CreateCoreButton(f)
	f.main:SetPoint("TOPLEFT", f, "TOPLEFT", 5, -15)
	--[[
	local bg = f:CreateTexture()
	bg:SetAllPoints()
	bg:SetTexture(0.5, 1, 0.5, 1)
	]]
	f.Update = Update
	f.Remove = Remove
	f.SetName = SetName
	f.UpdateState = UpdateState
	f.SetDescription = SetDescription
	f.UpdateColor = UpdateColor
	
	C.colorFrames[#C.colorFrames+1] = f
	
	return f
end
	
C.prototypes["color"] = "CreateColorFrame"